function y_3=calcRMSerror(time, D_sel, phi_sel, pulse, dpulse, sigma_2, VAofBasestation, VAofAgent, floorplan, AntennaSelektor,alpha)
% computes the Fisher information matrix for agent position.
%
% [1] Josef Kulmer, Stefan Grebien, Michael Rath, Klaus Witrisal: 
%      Hybrid analog-digital beamforming for single-anchor
%      multipath-assisted positioning, WCNC 2018


[NMPC,NAnt]=size(D_sel);

N=length(pulse);
c=3e8;
% note, the pulse derivative reaches rather large values due to the high sampling
% frequency, which subsequently results in a condition number of ~10^30
% Hence, we divide the sampling frequency and speed of light by a factor of
% 10^9
korrekturfaktor=1e9;
time.Ts=time.Ts*korrekturfaktor;
c=c/korrekturfaktor;
dpulse=dpulse/korrekturfaktor;

Pulse=fft(pulse);
dPulse=fft(dpulse);

PulseStore=zeros(N,NMPC);
dPulseStore=zeros(N,NMPC);

% store shifted pulse and dpulse
iAnt=1;
for iMPC=1:NMPC
    linphaseg=D_sel(iMPC,iAnt)/c/time.Ts;
    linphase=(-(time.N_sig_samp-1)/2 : (time.N_sig_samp-1)/2)'/time.N_sig_samp*2*pi*linphaseg;
    PulseStore(:,iMPC)=ifft(Pulse.*exp(-1i*ifftshift(linphase)));
    dPulseStore(:,iMPC)=ifft(dPulse.*exp(-1i*ifftshift(linphase)));
end

% % get beampatterns for MPCs and Antennas
for iMPC=1:NMPC
    b(iMPC,:) = ((cos(phi_sel(iMPC,:))/2+0.5).^2+0.05)/1.05; % parametrized antenna pattern
    db(iMPC,:) = -0.47619*sin(phi_sel(iMPC,:)).*(cos(phi_sel(iMPC,:))+1);
end

% correct dimension of alpha;
alpha=alpha.';

I=zeros(4*NMPC,4*NMPC);
% Fisher information for delays, angles and alphas
% we have four blocks with length NMPC
for iAnt=AntennaSelektor
    for iMPC=1:NMPC
        for jMPC=1:NMPC
        % block 1x1 (tau x tau)
            I(iMPC,jMPC) = I(iMPC,jMPC)...
                +1/sigma_2*2*real(conj(b(iMPC,iAnt))*conj(alpha(iMPC,iAnt))...
                *dPulseStore(:,iMPC)'*dPulseStore(:,jMPC)*b(jMPC,iAnt)*alpha(jMPC,iAnt));
        % block 1x2 (tau x phi)
            I(iMPC,jMPC+NMPC) = I(iMPC,jMPC+NMPC)...
                +1/sigma_2*2*real(conj(b(iMPC,iAnt))*conj(alpha(iMPC,iAnt))...
                *dPulseStore(:,iMPC)'*PulseStore(:,jMPC)*db(jMPC,iAnt)*alpha(jMPC,iAnt));
        % block 1x3 (tau x real(alpha))
            I(iMPC,jMPC+2*NMPC) = I(iMPC,jMPC+2*NMPC)...
                +1/sigma_2*2*real(conj(b(iMPC,iAnt))*conj(alpha(iMPC,iAnt))...
                *dPulseStore(:,iMPC)'*PulseStore(:,jMPC)*b(jMPC,iAnt));
        % block 1x4 (tau x imag(alpha))
            I(iMPC,jMPC+3*NMPC) = I(iMPC,jMPC+3*NMPC)...
                +1/sigma_2*2*real(conj(b(iMPC,iAnt))*conj(alpha(iMPC,iAnt))...
                *dPulseStore(:,iMPC)'*PulseStore(:,jMPC)*b(jMPC,iAnt)*1i);
        % block 2x2 (phi x phi)
            I(iMPC+1*NMPC,jMPC+1*NMPC) = I(iMPC+1*NMPC,jMPC+1*NMPC)...
                +1/sigma_2*2*real(conj(db(iMPC,iAnt))*conj(alpha(iMPC,iAnt))...
                *PulseStore(:,iMPC)'*PulseStore(:,jMPC)*db(jMPC,iAnt)*alpha(jMPC,iAnt));
        % block 2x3 (phi x real(alpha))
            I(iMPC+1*NMPC,jMPC+2*NMPC) = I(iMPC+1*NMPC,jMPC+2*NMPC)...
                +1/sigma_2*2*real(conj(db(iMPC,iAnt))*conj(alpha(iMPC,iAnt))...
                *PulseStore(:,iMPC)'*PulseStore(:,jMPC)*b(jMPC,iAnt));
        % block 2x4 (phi x imag(alpha))
            I(iMPC+1*NMPC,jMPC+3*NMPC) = I(iMPC+1*NMPC,jMPC+3*NMPC)...
                +1/sigma_2*2*real(conj(db(iMPC,iAnt))*conj(alpha(iMPC,iAnt))...
                *PulseStore(:,iMPC)'*PulseStore(:,jMPC)*b(jMPC,iAnt)*1i);
        % block 3x3 (real(alpha) x real(alpha))
            I(iMPC+2*NMPC,jMPC+2*NMPC) = I(iMPC+2*NMPC,jMPC+2*NMPC)...
                +1/sigma_2*2*real(conj(b(iMPC,iAnt))...
                *PulseStore(:,iMPC)'*PulseStore(:,jMPC)*b(jMPC,iAnt));
        % block 3x4 (real(alpha) x imag(alpha))
            I(iMPC+2*NMPC,jMPC+3*NMPC) = I(iMPC+2*NMPC,jMPC+3*NMPC)...
                +1/sigma_2*2*real(conj(b(iMPC,iAnt))...
                *PulseStore(:,iMPC)'*PulseStore(:,jMPC)*b(jMPC,iAnt)*1i);
        % block 4x4 (imag(alpha) x imag(alpha))
            I(iMPC+3*NMPC,jMPC+3*NMPC) = I(iMPC+3*NMPC,jMPC+3*NMPC)...
                +1/sigma_2*2*real(conj(b(iMPC,iAnt))*conj(1i)...
                *PulseStore(:,iMPC)'*PulseStore(:,jMPC)*b(jMPC,iAnt)*1i);
        end
    end
end

% make I symmetric
for iMPC=1:NMPC*4
    I(iMPC:end,iMPC)=I(iMPC,iMPC:end);
end

% Variable transformation from delays and angles to positions
J=zeros(2,NMPC*4); % create Jacobian
for iMPC=1:NMPC
    J(1,iMPC)=-(VAofBasestation.VA(1,iMPC)-VAofAgent.VA(1,1))/D_sel(iMPC,iAnt) /c;
    J(2,iMPC)=-(VAofBasestation.VA(2,iMPC)-VAofAgent.VA(2,1))/D_sel(iMPC,iAnt) /c;
    if iMPC==1 % LOS
        gradient=[0,-1;1,0]*(VAofAgent.VA([2,1],1)-VAofBasestation.VA([2,1],iMPC));
    else % Fantastic!
        gradient=-(eye(2)-2*floorplan.Rs{iMPC-1})*[0,-1;1,0]*(VAofAgent.VA([2,1],iMPC)-VAofBasestation.VA([2,1],1));
    end
    J(1,iMPC+NMPC)=gradient(1)/D_sel(iMPC,iAnt)^2;
    J(2,iMPC+NMPC)=gradient(2)/D_sel(iMPC,iAnt)^2;
end
% do variable transformation
crlb_pos=inv(J*I*J');
y_3=sqrt(sum(diag(crlb_pos)));
