function [rc, f_meas] = rcos_win(md, f)
%
% This function computes the frequency response of a raised-cosine
% with roll-off factor md.beta and pulse duration md.Tp. 1/Tp is 
% the 3dB bandwidth of the filter. md.fc is the center frequency and f
% the measured frequency vector (to check if in measured BW).
%
% rc is the transfer function of the rcos filter
% f_meas is the frequency vector where the rcos filter is non-zero,
% i.e. the range that will be cut out of the f vector.

beta = md.beta;
Tp = md.Tp;
fc = md.fc;

f_res = f(2) - f(1);
f_u = fc + (1+beta)/(2*Tp);  
f_l = fc - (1+beta)/(2*Tp);

if(f_u > max(f) || f_l < min(f))
  error('Selected bandwidth exceeds measured bandwidth');
end

f_meas = f( (f >= (f_l - f_res/2)) & (f <= (f_u + f_res/2) ));   %cut out selected BW
%f_res/2 for rounding issues
N = length(f_meas);

rc = zeros(N,1);

%Shift frequency vector to baseband for pulse computation
f_meas = f_meas-fc;

%Construct raised cosine frequency response piecewise
rc_1 = ones(N,1)*Tp;
rc_2 = Tp/2 * ( 1 + cos( pi*Tp/beta*(abs(f_meas) - (1-beta)/(2*Tp)) ) );
rc_3 = zeros(N,1);

idx_1 = find( (abs(f_meas) >= 0) & ( abs(f_meas) <= (1-beta)/(2*Tp) ) );
idx_2 = find( ( abs(f_meas) >= (1-beta)/(2*Tp) ) & ( abs(f_meas) <= (1+beta)/(2*Tp) ) );
idx_3 = find(abs(f_meas) > (1+beta)/(2*Tp) );

rc(idx_1) = rc_1(idx_1);
rc(idx_2) = rc_2(idx_2);
rc(idx_3) = rc_3(idx_3);

%Normalize to unit ampitude
rc = rc./max(rc);
f_meas = f_meas+fc;
f_meas = f_meas(:);
